
# q-expansion of newform 6300.2.k.g, downloaded from the LMFDB on 12 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6300
weight = 2
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[3151, [1, 0]], [2801, [1, 0]], [3277, [-1, 0]], [3601, [1, 0]]]
aps_data = [[0, 0], [0, 0], [0, 0], [0, -1], [-2, 0], [0, 6], [0, 4], [4, 0], [0, 2], [-2, 0], [0, 0], [0, 2], [0, 0], [0, 4], [0, -12], [0, -6], [-8, 0], [6, 0], [0, -8], [-14, 0], [0, 2], [-12, 0], [0, -4], [0, 0], [0, -2], [-16, 0], [0, 16], [0, -18], [2, 0], [0, 10], [0, 12], [-4, 0], [0, 2], [-4, 0], [-6, 0], [-8, 0], [0, 14], [0, 16], [0, -4], [0, 16], [-18, 0], [-6, 0], [18, 0], [0, 10], [0, 22], [16, 0], [-4, 0], [0, -8], [0, 0], [-10, 0], [0, 14], [-6, 0], [14, 0], [24, 0], [0, 24], [0, 18], [-20, 0], [24, 0], [0, -22], [10, 0], [0, -4], [0, 0], [0, 4], [-4, 0], [0, -10], [0, -18], [-28, 0], [0, -6], [0, -6], [-22, 0], [0, 12], [-2, 0], [0, -16], [0, -22], [-4, 0], [0, 8], [22, 0], [0, -18], [18, 0], [-38, 0], [0, 0], [-6, 0], [-30, 0], [0, -26], [24, 0], [0, 38], [-14, 0], [0, 10], [-40, 0], [0, -4], [0, 8], [-28, 0], [0, -16], [42, 0], [20, 0], [0, -24], [-36, 0], [-12, 0], [0, 28], [-30, 0], [0, 24], [0, 38], [0, 8], [30, 0], [-24, 0], [0, -38], [0, -32], [0, 12], [6, 0], [-46, 0], [0, 0], [0, -6], [0, -38], [20, 0], [20, 0], [10, 0], [0, -44], [0, -12], [0, 6], [2, 0], [22, 0], [0, 34], [0, 24], [0, 30], [-28, 0], [50, 0], [-38, 0], [0, 0], [0, 16], [0, 6], [0, 0], [0, 30], [0, 0], [0, -42], [-32, 0], [38, 0], [0, -8], [0, -12], [0, 0], [10, 0], [28, 0], [-2, 0], [0, 20], [0, -18], [14, 0], [44, 0], [0, -14], [0, 48], [-44, 0], [0, -14], [0, 46], [-36, 0], [0, -44], [0, 12], [0, 12], [18, 0], [-16, 0], [48, 0], [0, 2], [-28, 0], [0, -10], [0, -22], [0, 20], [-36, 0], [0, -30], [0, -48], [56, 0], [0, 38], [-14, 0], [0, 0], [-38, 0], [-50, 0], [42, 0], [0, -6], [-24, 0], [4, 0], [48, 0], [-38, 0], [0, 40], [34, 0], [0, 36], [-20, 0], [0, 38], [0, 20], [0, -62], [24, 0], [0, 6], [0, -44], [42, 0], [24, 0], [0, -46], [0, 18], [52, 0], [-8, 0], [0, 30], [0, -28], [-34, 0], [0, 62], [0, 0], [0, -36], [-30, 0], [-24, 0], [0, 2], [18, 0], [60, 0], [0, 0], [-8, 0], [0, 42], [-6, 0], [-52, 0], [0, 2], [20, 0], [0, 48], [0, -4], [36, 0], [50, 0], [0, -8], [-20, 0], [0, -2], [0, 14], [22, 0], [-16, 0], [42, 0], [0, -16], [0, -36], [-50, 0], [0, 32], [58, 0], [0, 48], [66, 0], [0, -30], [44, 0], [-8, 0], [6, 0], [0, 4], [0, 0], [2, 0], [0, 66], [-18, 0], [-40, 0], [0, -10], [44, 0], [0, 16], [30, 0], [0, -64], [36, 0], [0, -48], [-32, 0], [-8, 0], [0, 18], [0, 70], [36, 0], [0, 42], [50, 0], [0, -16], [50, 0], [26, 0], [0, -36], [0, -40], [0, 50], [0, -76], [0, 74], [-10, 0], [0, -22], [0, 60], [44, 0], [42, 0], [-28, 0], [0, -64], [0, 54], [-42, 0], [0, -28], [0, 18], [-8, 0], [0, -26], [0, 48], [0, 22], [-66, 0], [46, 0], [48, 0], [0, 24], [-60, 0], [0, 48], [-10, 0], [0, 36], [-62, 0], [0, -18], [0, -2], [16, 0], [24, 0], [-18, 0], [0, -84], [0, 54], [32, 0], [0, 34], [0, 0], [8, 0], [0, -36], [24, 0], [0, -36], [0, -10], [0, 18], [44, 0], [0, 66], [8, 0], [0, -70], [0, 50], [46, 0], [-34, 0], [0, 22], [0, 24], [-50, 0], [50, 0], [0, 4], [0, 62], [34, 0], [36, 0], [-10, 0], [0, 10], [54, 0], [-4, 0], [0, 22], [-24, 0], [0, 56], [0, -62], [38, 0], [40, 0], [0, -4], [0, 58], [0, 86], [38, 0], [0, 10], [-72, 0], [0, 32], [-32, 0], [0, 8], [-30, 0], [0, 40], [86, 0], [0, -56], [0, 10], [0, 42], [12, 0], [48, 0], [0, 78], [-30, 0], [-46, 0], [0, -52], [-72, 0], [0, 48], [-76, 0], [0, 38], [-18, 0], [0, -24], [74, 0], [0, -28], [16, 0], [12, 0], [0, 70], [0, 38], [0, 22], [40, 0], [0, -46], [26, 0], [0, 52], [0, 10], [0, -84], [0, -36], [94, 0], [58, 0], [-76, 0], [0, -90], [-62, 0], [-80, 0], [0, 34], [-60, 0], [22, 0], [0, 38], [44, 0], [0, 14], [72, 0], [0, -38], [0, -32], [0, 6], [-28, 0], [0, 64], [80, 0], [0, 82], [0, 32], [0, 28], [-62, 0], [0, 36], [2, 0], [0, 68], [-94, 0], [0, -66], [8, 0], [20, 0], [48, 0], [-62, 0], [58, 0], [0, -6], [0, 28], [0, -16], [84, 0], [8, 0], [0, -14], [78, 0], [0, 52], [-16, 0], [0, 62], [0, 58], [0, 14], [84, 0], [0, -58], [-20, 0], [-46, 0], [0, -8], [0, 60], [0, -92], [-34, 0], [0, -10], [0, -62], [-84, 0], [0, -26], [0, -16], [0, 22], [54, 0], [84, 0], [-76, 0]]
